@@ -14,7 +14,7 @@ from datetime import datetime
 
 
 from pelican import signals
-from pelican.settings import _DEFAULT_CONFIG
+from pelican.settings import DEFAULT_CONFIG
 from pelican.utils import (slugify, truncate_html_words, memoized, strftime,
                            python_2_unicode_compatible, deprecated_attribute,
                            path_to_url)
@@ -44,14 +44,16 @@ class Content(object):
         if metadata is None:
             metadata = {}
         if settings is None:
-            settings = copy.deepcopy(_DEFAULT_CONFIG)
+            settings = copy.deepcopy(DEFAULT_CONFIG)
 
         self.settings = settings
         self._content = content
+        if context is None:
+            context = {}
         self._context = context
         self.translations = []
 
-        local_metadata = dict(settings.get('DEFAULT_METADATA', ()))
+        local_metadata = dict(settings['DEFAULT_METADATA'])
         local_metadata.update(metadata)
 
         # set metadata as attributes
@@ -84,7 +86,8 @@ class Content(object):
 
         # create the slug if not existing, from the title
         if not hasattr(self, 'slug') and hasattr(self, 'title'):
-            self.slug = slugify(self.title)
+            self.slug = slugify(self.title,
+                                settings.get('SLUG_SUBSTITUTIONS', ()))
 
         self.source_path = source_path
 
@@ -138,15 +141,22 @@ class Content(object):
         """Returns the URL, formatted with the proper values"""
         metadata = copy.copy(self.metadata)
         path = self.metadata.get('path', self.get_relative_source_path())
+        default_category = self.settings['DEFAULT_CATEGORY']
+        slug_substitutions = self.settings.get('SLUG_SUBSTITUTIONS', ())
         metadata.update({
             'path': path_to_url(path),
             'slug': getattr(self, 'slug', ''),
             'lang': getattr(self, 'lang', 'en'),
             'date': getattr(self, 'date', datetime.now()),
-            'author': getattr(self, 'author', ''),
-            'category': getattr(self, 'category',
-                self.settings['DEFAULT_CATEGORY']),
-            })
+            'author': slugify(
+                getattr(self, 'author', ''),
+                slug_substitutions
+            ),
+            'category': slugify(
+                getattr(self, 'category', default_category),
+                slug_substitutions
+            )
+        })
         return metadata
 
     def _expand_settings(self, key):
@@ -169,13 +179,18 @@ class Content(object):
         :param siteurl: siteurl which is locally generated by the writer in
                         case of RELATIVE_URLS.
         """
-        hrefs = re.compile(r"""
+        if not content:
+            return content
+
+        instrasite_link_regex = self.settings['INTRASITE_LINK_REGEX']
+        regex = r"""
             (?P<markup><\s*[^\>]*  # match tag with src and href attr
                 (?:href|src)\s*=)
 
             (?P<quote>["\'])      # require value to be quoted
-            (?P<path>\|(?P<what>.*?)\|(?P<value>.*?))  # the url value
-            \2""", re.X)
+            (?P<path>{0}(?P<value>.*?))  # the url value
+            \2""".format(instrasite_link_regex)
+        hrefs = re.compile(regex, re.X)
 
         def replacer(m):
             what = m.group('what')
@@ -202,7 +217,11 @@ class Content(object):
                     origin = origin.replace('\\', '/')  # Fow windows paths.
                 else:
                     logger.warning("Unable to find {fn}, skipping url"
-                                    " replacement".format(fn=value))
+                                   " replacement".format(fn=value))
+            elif what == 'category':
+                origin = Category(value, self.settings).url
+            elif what == 'tag':
+                origin = Tag(value, self.settings).url
 
             return ''.join((m.group('markup'), m.group('quote'), origin,
                             m.group('quote')))
@@ -220,7 +239,7 @@ class Content(object):
 
     @property
     def content(self):
-        return self.get_content(self._context['localsiteurl'])
+        return self.get_content(self._context.get('localsiteurl', ''))
 
     def _get_summary(self):
         """Returns the summary of an article.
@@ -231,10 +250,11 @@ class Content(object):
         if hasattr(self, '_summary'):
             return self._summary
 
-        if self.settings['SUMMARY_MAX_LENGTH']:
-            return truncate_html_words(self.content,
-                    self.settings['SUMMARY_MAX_LENGTH'])
-        return self.content
+        if self.settings['SUMMARY_MAX_LENGTH'] is None:
+            return self.content
+
+        return truncate_html_words(self.content,
+                                   self.settings['SUMMARY_MAX_LENGTH'])
 
     def _set_summary(self, summary):
         """Dummy function"""
@@ -311,5 +331,5 @@ def is_valid_content(content, f):
         return True
     except NameError as e:
         logger.error("Skipping %s: could not find information about "
-                      "'%s'" % (f, e))
+                     "'%s'" % (f, e))
         return False
