@@ -6,11 +6,15 @@ import os
 import locale
 import logging
 
-from codecs import open
+if not six.PY3:
+    from codecs import open
+
 from feedgenerator import Atom1Feed, Rss201rev2Feed
 from jinja2 import Markup
+
 from pelican.paginator import Paginator
 from pelican.utils import get_relative_path, path_to_url, set_date_tzinfo
+from pelican import signals
 
 logger = logging.getLogger(__name__)
 
@@ -21,6 +25,8 @@ class Writer(object):
         self.output_path = output_path
         self.reminder = dict()
         self.settings = settings or {}
+        self._written_files = set()
+        self._overridden_files = set()
 
     def _create_new_feed(self, feed_type, context):
         feed_class = Rss201rev2Feed if feed_type == 'rss' else Atom1Feed
@@ -46,6 +52,29 @@ class Writer(object):
             pubdate=set_date_tzinfo(item.date,
                 self.settings.get('TIMEZONE', None)))
 
+    def _open_w(self, filename, encoding, override=False):
+        """Open a file to write some content to it.
+
+        Exit if we have already written to that file, unless one (and no more
+        than one) of the writes has the override parameter set to True.
+        """
+        if filename in self._overridden_files:
+            if override:
+                raise StandardError('File %s is set to be overridden twice'
+                                    % filename)
+            else:
+                logger.info('skipping %s' % filename)
+                filename = os.devnull
+        elif filename in self._written_files:
+            if override:
+                logger.info('overwriting %s' % filename)
+            else:
+                raise StandardError('File %s is to be overwritten' % filename)
+        if override:
+            self._overridden_files.add(filename)
+        self._written_files.add(filename)
+        return open(filename, 'w', encoding=encoding)
+
     def write_feed(self, elements, context, path=None, feed_type='atom'):
         """Generate a feed with the list of articles provided
 
@@ -82,7 +111,7 @@ class Writer(object):
                     pass
 
                 encoding = 'utf-8' if six.PY3 else None
-                with open(complete_path, 'w', encoding=encoding) as fp:
+                with self._open_w(complete_path, encoding) as fp:
                     feed.write(fp, 'utf-8')
                     logger.info('writing %s' % complete_path)
             return feed
@@ -90,7 +119,7 @@ class Writer(object):
             locale.setlocale(locale.LC_ALL, old_locale)
 
     def write_file(self, name, template, context, relative_urls=False,
-        paginated=None, **kwargs):
+        paginated=None, override_output=False, **kwargs):
         """Render the template and write the file.
 
         :param name: name of the file to output
@@ -99,6 +128,9 @@ class Writer(object):
         :param relative_urls: use relative urls or absolutes ones
         :param paginated: dict of article list to paginate - must have the
             same length (same list in different orders)
+        :param override_output: boolean telling if we can override previous
+            output with the same name (and if next files written with the same
+            name should be skipped to keep that one)
         :param **kwargs: additional variables to pass to the templates
         """
 
@@ -108,7 +140,7 @@ class Writer(object):
             # other stuff, just return for now
             return
 
-        def _write_file(template, localcontext, output_path, name):
+        def _write_file(template, localcontext, output_path, name, override):
             """Render the template write the file."""
             old_locale = locale.setlocale(locale.LC_ALL)
             locale.setlocale(locale.LC_ALL, str('C'))
@@ -121,10 +153,15 @@ class Writer(object):
                 os.makedirs(os.path.dirname(path))
             except Exception:
                 pass
-            with open(path, 'w', encoding='utf-8') as f:
+
+            with self._open_w(path, 'utf-8', override=override) as f:
                 f.write(output)
             logger.info('writing {}'.format(path))
 
+            # Send a signal to say we're writing a file with some specific
+            # local context.
+            signals.content_written.send(path, context=localcontext)
+
         localcontext = context.copy()
         if relative_urls:
             relative_url = path_to_url(get_relative_path(name))
@@ -137,36 +174,38 @@ class Writer(object):
         # check paginated
         paginated = paginated or {}
         if paginated:
+            name_root = os.path.splitext(name)[0]
+
             # pagination needed, init paginators
             paginators = {}
             for key in paginated.keys():
                 object_list = paginated[key]
 
-                if self.settings.get('DEFAULT_PAGINATION'):
-                    paginators[key] = Paginator(object_list,
-                        self.settings.get('DEFAULT_PAGINATION'),
-                        self.settings.get('DEFAULT_ORPHANS'))
-                else:
-                    paginators[key] = Paginator(object_list, len(object_list))
+                paginators[key] = Paginator(
+                    name_root,
+                    object_list,
+                    self.settings,
+                )
 
             # generated pages, and write
-            name_root, ext = os.path.splitext(name)
             for page_num in range(list(paginators.values())[0].num_pages):
                 paginated_localcontext = localcontext.copy()
                 for key in paginators.keys():
                     paginator = paginators[key]
+                    previous_page = paginator.page(page_num) \
+                            if page_num > 0 else None
                     page = paginator.page(page_num + 1)
+                    next_page = paginator.page(page_num + 2) \
+                            if page_num + 1 < paginator.num_pages else None
                     paginated_localcontext.update(
                             {'%s_paginator' % key: paginator,
-                             '%s_page' % key: page})
-                if page_num > 0:
-                    paginated_name = '%s%s%s' % (
-                        name_root, page_num + 1, ext)
-                else:
-                    paginated_name = name
+                             '%s_page' % key: page,
+                             '%s_previous_page' % key: previous_page,
+                             '%s_next_page' % key: next_page})
 
                 _write_file(template, paginated_localcontext, self.output_path,
-                    paginated_name)
+                            page.save_as, override_output)
         else:
             # no pagination
-            _write_file(template, localcontext, self.output_path, name)
+            _write_file(template, localcontext, self.output_path, name,
+                        override_output)
